/*
 * Preferences dialog based on ReplaceDialog.java
 */

// Current elements are:
//
//   [] Automatically insert close-parentheses
//   <-|--> Relative height of completion box
//
package jnyqide;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.sun.corba.se.spi.servicecontext.SendingContextServiceContext;

import jnyqide.*;

class PreferencesDialog extends JInternalFrame implements ActionListener {
    private JButton defaultPrefs;
    private JCheckBox closeParenCheckbox;
    private JScrollBar completionListPercent;
    private MainFrame mainFrame;
    private JComboBox audioRate;
    private String currentAudioRate;
    private JTextField controlRate;
    private JCheckBox autoNorm;
    private JCheckBox backTrace;
    private JCheckBox printGC;
    private JCheckBox soundEnable;
    private JButton defDir;
    private JFileChooser fd;
    private JCheckBox fullSearchCheckbox;
    private JCheckBox internalBrowser;
    private JCheckBox onlineManual;
    private JCheckBox startWithSal;
    private JCheckBox salShowLisp;
    private JCheckBox salTraceback;
    private String[] audioRates = { "96000", "48000", "44100", "22050", "16000",
                                    "11025", "8000" };
                                    
    protected JCheckBox makeCheckBox(String doc, boolean val) {
        JPanel contentPane = (JPanel) getContentPane();
        JCheckBox cb = new JCheckBox(doc, val);
        cb.setAlignmentX(Component.LEFT_ALIGNMENT);
        contentPane.add(cb);
        return cb;
    }
                                    
    public PreferencesDialog(MainFrame mainFrame_) {
        super();
        final PreferencesDialog preferencesDialog = this;
        //super(mainFrame_, "Preferences", true); // initialize Dialog
        setTitle("Preferences");

        mainFrame = mainFrame_;
                
        addInternalFrameListener(new InternalFrameListener() {
            public void internalFrameClosing(InternalFrameEvent e) {
                mainFrame.prefParenAutoInsert = closeParenCheckbox.isSelected();
                
                if ((int) mainFrame.prefCompletionListPercent != 
                    (int) (completionListPercent.getValue())) {
                    mainFrame.prefCompletionListPercent = 
                            completionListPercent.getValue();
                    mainFrame.tileCompletion();
                }
                
                String rateString = (String) audioRate.getSelectedItem();
                audioRate.setSize(50, 20);
                int rate = validate(currentAudioRate);
                if (rate > 0 && !audioRate.equals(mainFrame.prefAudioRate)) {
                    mainFrame.callFunction("set-sound-srate", 
                                           String.valueOf(rate));
                    mainFrame.prefAudioRate = rateString;
                }

                boolean enable = soundEnable.isSelected();
		if (enable != mainFrame.prefSoundEnable) {
                    mainFrame.callFunction(
                            enable ? "sound-off" : "sound-on", "");
                    mainFrame.prefSoundEnable = enable;
                }
                
                rateString = controlRate.getText();
		rate = validate(rateString);
		if (rate > 0 && !controlRate.equals(mainFrame.prefControlRate)) {
                    mainFrame.callFunction("set-control-srate ", 
                                           String.valueOf(rate));
                    mainFrame.prefControlRate = rateString;
		}
                
                enable = autoNorm.isSelected();
		if (enable != mainFrame.prefAutoNorm) {
                    mainFrame.callFunction(
                            enable ? "autonorm-on" : "autonorm-off", "");
                    mainFrame.prefAutoNorm = enable;
                }

                enable = backTrace.isSelected();
                if (enable != mainFrame.prefPrintStackTrace) {
                    mainFrame.setBoolean("*tracenable*", enable);
                    mainFrame.prefPrintStackTrace = enable;
                }
                
                enable = printGC.isSelected();
                if (enable != mainFrame.prefPrintGC) {
                    mainFrame.setBoolean("*gc-flag*", enable);
                    mainFrame.prefPrintGC = enable;
                }
                
                String dir = fd.getSelectedFile().toString();
                System.out.println("fd.getSelectedFile: " + dir);
		if (dir != null && dir.length() > 0) {
                    mainFrame.prefDirectory = dir;
                }
                        
                mainFrame.prefFullSearch = fullSearchCheckbox.isSelected();
                mainFrame.prefInternalBrowser = internalBrowser.isSelected();
                mainFrame.prefOnlineManual = onlineManual.isSelected();
                mainFrame.prefStartWithSal = startWithSal.isSelected();
                
                enable = salShowLisp.isSelected();
                if (enable != mainFrame.prefSalShowLisp) {
                    mainFrame.setBoolean("*sal-compiler-debug*", enable);
                    mainFrame.prefSalShowLisp = enable;
                }
                
                enable = salTraceback.isSelected();
                if (enable != mainFrame.prefSalTraceback) {
                    mainFrame.setBoolean("*tracenable*", enable);
                    mainFrame.prefSalTraceback = enable;
                }
                
                mainFrame.prefsHaveBeenSet = true;
                dispose();
            }

            public void internalFrameOpened(InternalFrameEvent e) {
            }
            public void internalFrameClosed(InternalFrameEvent e) {
                mainFrame.disconnectPreferences();
                //System.out.println("FrameClosed");
            }
            public void internalFrameIconified(InternalFrameEvent e) {
            }
            public void internalFrameDeiconified(InternalFrameEvent e) {
            }
            public void internalFrameActivated(InternalFrameEvent e) {
            }
            public void internalFrameDeactivated(InternalFrameEvent e) {
            }
        });

        JPanel contentPane = (JPanel) getContentPane();
        contentPane.setLayout(
                new BoxLayout(contentPane, BoxLayout.Y_AXIS));
                
        // button to restore default preferences
        defaultPrefs = new JButton("Restore Defaults");
        defaultPrefs.addActionListener(this);
        defaultPrefs.setAlignmentX(Component.LEFT_ALIGNMENT);
        contentPane.add(defaultPrefs);

        contentPane.add(Box.createRigidArea(new Dimension(0, 10)));
        
        // Start in Sal mode
        startWithSal = makeCheckBox("Start in SAL mode (not Lisp)", 
                                    mainFrame.prefStartWithSal);
        // Show Lisp after compiling Sal
        salShowLisp = makeCheckBox("Show translation of SAL to Lisp",
                                   mainFrame.prefSalShowLisp);
        // Enable SAL Stack Traceback on Error
        salTraceback = makeCheckBox("Show SAL stack traceback on error",
                                    mainFrame.prefSalTraceback);
        // Automatically insert close-parenthesis (checkbox)
        closeParenCheckbox = makeCheckBox(
                                "Automatically insert close-parentheses",
                                mainFrame.prefParenAutoInsert);
        // Enable sound output (checkbox)
        soundEnable = makeCheckBox("Enable sound output in PLAY command",
                                   mainFrame.prefSoundEnable);
        // AutoNorm (checkbox)
        autoNorm = makeCheckBox("AutoNorm", mainFrame.prefAutoNorm);
        
        // printStackTrace
        backTrace = makeCheckBox("Print Stack Trace on Error",
                                 mainFrame.prefPrintStackTrace);
        // printGC
        printGC = makeCheckBox("Print info about garbage collection",
                               mainFrame.prefPrintGC);

        // Use full search for code completion (checkbox)
        fullSearchCheckbox = makeCheckBox("Use full search for code completion",
                                          mainFrame.prefFullSearch);
        // Use internal window for manual (checkbox)
        internalBrowser = makeCheckBox("Use window in jNyqIDE for help browser",
                                       mainFrame.prefInternalBrowser);
        // Use online manual (checkbox)
        onlineManual = makeCheckBox("Use online manual instead of local copy",
                                    mainFrame.prefOnlineManual);

        contentPane.add(Box.createRigidArea(new Dimension(0, 10)));

        // Relative height of completion box (slider)
        contentPane.add(new JLabel("Relative height of completion box", 
                                    JLabel.CENTER));
        completionListPercent = new JScrollBar(JScrollBar.HORIZONTAL,
                    (int) mainFrame.prefCompletionListPercent, 1, 0, 100);
        contentPane.add(completionListPercent);

        contentPane.add(Box.createRigidArea(new Dimension(0, 10)));

        // Audio Sample Rate (editable combobox)
        contentPane.add(new JLabel("Audio Sample Rate")); 
        audioRate = new JComboBox(audioRates);
        // Set correct selection
        for (int i = 0; i < audioRates.length; i++) {
            if (mainFrame.prefAudioRate.equals(audioRates[i])) {
                audioRate.setSelectedIndex(i);
                break;
            }
        }
        audioRate.setEditable(true);
        audioRate.setAlignmentX(Component.LEFT_ALIGNMENT);
        audioRate.setMaximumSize(
                new Dimension(100, audioRate.getPreferredSize().height));
        currentAudioRate = mainFrame.prefAudioRate;
        contentPane.add(audioRate);
            
        contentPane.add(Box.createRigidArea(new Dimension(0, 10)));

        // Control Rate (text field)
        contentPane.add(new JLabel("Control Sample Rate")); 
        controlRate = new JTextField(mainFrame.prefControlRate);
        controlRate.setAlignmentX(Component.LEFT_ALIGNMENT);
        controlRate.setMaximumSize(
                new Dimension(100, controlRate.getPreferredSize().height));
        contentPane.add(controlRate);

        contentPane.add(Box.createRigidArea(new Dimension(0, 10)));

        // Select Output Directory (button)
        fd = new JFileChooser("Select Output Direcotry");
        fd.setSelectedFile(mainFrame.homeDir);
        fd.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        defDir = new JButton("Set Default Output Direcotry");
        defDir.addActionListener(this);
        defDir.setAlignmentX(Component.LEFT_ALIGNMENT);
        contentPane.add(defDir);
        
        contentPane.add(Box.createVerticalGlue());
                
        /*
        * The Nyquist IDE has a preferences dialog with a couple of things you
        * can change. It would be great to have a graphical way to set things
        * like the normalization style, default audio and control sample rates,
        * whether to play sounds or save audio to disk or both when calling the
        * play function, whether to apply reverb and/or EQ to the output signal
        * when using PLAY, a default sound file directory, whether to print a
        * stack trace when an error is encountered, etc. (All of these things
        * can be set in Nyquist, but most users do not know how.)
        */

        pack();
        Dimension size = new Dimension(400, 500);
        setSize(size);
        Point mfloc = mainFrame.getLocation();
        setLocation(mfloc.x + 25, mfloc.y + 25);
        setResizable(true);
        setVisible(true);
        setClosable(true);
        setMaximizable(true);
        setIconifiable(true);
        repaint();
    }

    private int validate(String s) {
        try {
            int temp = Integer.parseInt(s);
            if (temp > 0) return temp;
            } catch (Exception e) {
        }
        return -1;
    }
    
    public void actionPerformed(ActionEvent evt) {
        if (evt.getSource() == defDir) {
            fd.showDialog(this, "Select");
        } else if (evt.getSource() == defaultPrefs) {
            closeParenCheckbox.setSelected(true);
            startWithSal.setSelected(false);
            salShowLisp.setSelected(false);
            soundEnable.setSelected(true);
            autoNorm.setSelected(true);
            backTrace.setSelected(false);
            printGC.setSelected(true);
            fullSearchCheckbox.setSelected(false);
            internalBrowser.setSelected(false);
            onlineManual.setSelected(false);
            completionListPercent.setValue(60);
            audioRate.setSelectedItem(audioRates[2]);
            controlRate.setText("2205");
            fd.setSelectedFile(mainFrame.homeDir);
        }
    } 
}
